package com.sh.sdk.shareinstall_flutter;

import android.content.Context;
import android.content.Intent;
import android.net.Uri;

import android.text.TextUtils;

import androidx.annotation.NonNull;

import com.sh.sdk.shareinstall.ShareInstall;
import com.sh.sdk.shareinstall.listener.AppDeleteInstallParamsListener;
import com.sh.sdk.shareinstall.listener.AppGetInstallListener;
import com.sh.sdk.shareinstall.listener.AppGetWakeUpListener;
import com.sh.sdk.shareinstall.listener.OnReportRegisterListener;

import java.util.concurrent.atomic.AtomicBoolean;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/**
 * ShareinstallFlutterPlugin
 */
public class ShareinstallFlutterPlugin implements FlutterPlugin, MethodCallHandler, PluginRegistry.NewIntentListener {

    private static final String GET_WAKEUP = "getWakeUp";
    private static final String GET_INSTALL = "getInstall";
    private static final String REPORT_REGISTER = "reportRegister";
    private static final String DEL_CUSTOM_PARAMS = "delCustomParams";

    private MethodChannel methodChannel;
    private AtomicBoolean isResultCallBack = new AtomicBoolean(false);

    public ShareinstallFlutterPlugin() {
    }

    /**
     * Plugin registration.
     */
    public static void registerWith(Registrar registrar) {
        ShareinstallFlutterPlugin plugin = new ShareinstallFlutterPlugin();
        plugin.onAttachedToEngine(registrar.context(), registrar.messenger());
        registrar.addNewIntentListener(plugin);
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case GET_WAKEUP:
                getWakeUp(call, result);
                break;
            case GET_INSTALL:
                getInstall(result);
                break;
            case REPORT_REGISTER:
                reportRegister(result);
                break;
            case DEL_CUSTOM_PARAMS:
                delCustomParams(result);
                break;
            default:
                result.notImplemented();
        }
    }

    private void getWakeUp(MethodCall call, final Result result) {
        String data = (String) call.arguments;
        Intent intent = new Intent();
        if (!TextUtils.isEmpty(data)) {
            intent.setData(Uri.parse(data));
        }
        ShareInstall.getInstance().getWakeUpParams(intent, new AppGetWakeUpListener() {
            @Override
            public void onGetWakeUpFinish(String json) {
                resultCallBack(result, json);
            }
        });
    }

    private void getWakeUp(Intent intent) {
        ShareInstall.getInstance().getWakeUpParams(intent, new AppGetWakeUpListener() {
            @Override
            public void onGetWakeUpFinish(String json) {
                methodChannel.invokeMethod("wakeup", json);
            }
        });
    }


    private void getInstall(final Result result) {
        ShareInstall.getInstance().getInstallParams(new AppGetInstallListener() {
            @Override
            public void onGetInstallFinish(String json) {
                resultCallBack(result, json);
            }
        });
    }

    private void reportRegister(final Result result) {
        ShareInstall.getInstance().reportRegister(new OnReportRegisterListener() {
            @Override
            public void onSuccess() {
                resultCallBack(result, "reportRegister success");
            }

            @Override
            public void onError() {
                resultCallBack(result, "reportRegister error");
            }
        });
    }

    private void delCustomParams(final Result result) {
        if (isResultCallBack == null) {
            isResultCallBack = new AtomicBoolean(false);
        }
        isResultCallBack.set(false);
        ShareInstall.getInstance().deleteInstallParams(new AppDeleteInstallParamsListener() {
            @Override
            public void onDeleteInstallParamsComplete() {
                if (isResultCallBack.compareAndSet(false, true)) {
                    resultCallBack(result, "delCustomParams complete");
                }
            }
        });
    }

    private void resultCallBack(Result result, String data) {
        result.success(data);
    }

    @Override
    public boolean onNewIntent(Intent intent) {
        getWakeUp(intent);
        return false;
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        onAttachedToEngine(binding.getApplicationContext(), binding.getBinaryMessenger());
    }

    private void onAttachedToEngine(Context applicationContext, BinaryMessenger messenger) {
        methodChannel = new MethodChannel(messenger, "shareinstall_flutter");
        methodChannel.setMethodCallHandler(this);
        ShareInstall.getInstance().init(applicationContext);
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        methodChannel.setMethodCallHandler(null);
        methodChannel = null;
    }
}
